import type { TableProps as RcTableProps } from '../vc-table/Table';
import type { SpinProps } from '../spin';
import type { TooltipProps } from '../tooltip';
import type { TableRowSelection, GetRowKey, ColumnType, ColumnsType, TableCurrentDataSource, SorterResult, GetPopupContainer, TablePaginationConfig, SortOrder, TableLocale, FilterValue } from './interface';
import type { SizeType } from '../config-provider';
import type { PropType } from 'vue';
import type { DefaultRecordType } from '../vc-table/interface';
import type { ContextSlots } from './context';
export type { ColumnsType, TablePaginationConfig };
export interface TableProps<RecordType = DefaultRecordType> extends Omit<RcTableProps<RecordType>, 'transformColumns' | 'internalHooks' | 'internalRefs' | 'data' | 'columns' | 'scroll' | 'emptyText' | 'canExpandable' | 'onUpdateInternalRefs'> {
    dropdownPrefixCls?: string;
    dataSource?: RcTableProps<RecordType>['data'];
    columns?: ColumnsType<RecordType>;
    pagination?: false | TablePaginationConfig;
    loading?: boolean | SpinProps;
    size?: SizeType;
    bordered?: boolean;
    locale?: TableLocale;
    onChange?: (pagination: TablePaginationConfig, filters: Record<string, FilterValue | null>, sorter: SorterResult<RecordType> | SorterResult<RecordType>[], extra: TableCurrentDataSource<RecordType>) => void;
    rowSelection?: TableRowSelection<RecordType>;
    getPopupContainer?: GetPopupContainer;
    scroll?: RcTableProps<RecordType>['scroll'] & {
        scrollToFirstRowOnChange?: boolean;
    };
    sortDirections?: SortOrder[];
    showSorterTooltip?: boolean | TooltipProps;
}
export declare const tableProps: () => {
    prefixCls: {
        type: PropType<string>;
        default: any;
    };
    columns: {
        type: PropType<ColumnsType<DefaultRecordType>>;
        default: any;
    };
    rowKey: {
        type: PropType<string | GetRowKey<DefaultRecordType>>;
        default: any;
    };
    tableLayout: {
        type: PropType<import("../vc-table/interface").TableLayout>;
        default: any;
    };
    rowClassName: {
        type: PropType<string | import("../vc-table/interface").RowClassName<DefaultRecordType>>;
        default: any;
    };
    title: {
        type: PropType<import("../vc-table/interface").PanelRender<DefaultRecordType>>;
        default: any;
    };
    footer: {
        type: PropType<import("../vc-table/interface").PanelRender<DefaultRecordType>>;
        default: any;
    };
    id: {
        type: PropType<string>;
        default: any;
    };
    showHeader: {
        type: PropType<boolean>;
        default: any;
    };
    components: {
        type: PropType<import("../vc-table/interface").TableComponents<DefaultRecordType>>;
        default: any;
    };
    customRow: {
        type: PropType<import("../vc-table/interface").GetComponentProps<DefaultRecordType>>;
        default: any;
    };
    customHeaderRow: {
        type: PropType<import("../vc-table/interface").GetComponentProps<import("../vc-table/interface").ColumnType<DefaultRecordType>[]>>;
        default: any;
    };
    direction: {
        type: PropType<"ltr" | "rtl">;
        default: any;
    };
    expandFixed: {
        type: PropType<boolean>;
        default: any;
    };
    expandColumnWidth: {
        type: PropType<number>;
        default: any;
    };
    expandedRowKeys: {
        type: PropType<import("../vc-table/interface").Key[]>;
        default: import("../vc-table/interface").Key[];
    };
    defaultExpandedRowKeys: {
        type: PropType<import("../vc-table/interface").Key[]>;
        default: import("../vc-table/interface").Key[];
    };
    expandedRowRender: {
        type: PropType<import("../vc-table/interface").ExpandedRowRender<DefaultRecordType>>;
        default: any;
    };
    expandRowByClick: {
        type: PropType<boolean>;
        default: any;
    };
    expandIcon: {
        type: PropType<import("../vc-table/interface").RenderExpandIcon<DefaultRecordType>>;
        default: any;
    };
    onExpand: {
        type: PropType<(expanded: boolean, record: DefaultRecordType) => void>;
        default: any;
    };
    onExpandedRowsChange: {
        type: PropType<(expandedKeys: import("../vc-table/interface").Key[]) => void>;
        default: any;
    };
    defaultExpandAllRows: {
        type: PropType<boolean>;
        default: any;
    };
    indentSize: {
        type: PropType<number>;
        default: any;
    };
    expandIconColumnIndex: {
        type: PropType<number>;
        default: any;
    };
    expandedRowClassName: {
        type: PropType<import("../vc-table/interface").RowClassName<DefaultRecordType>>;
        default: any;
    };
    childrenColumnName: {
        type: PropType<string>;
        default: any;
    };
    rowExpandable: {
        type: PropType<(record: DefaultRecordType) => boolean>;
        default: any;
    };
    sticky: {
        type: PropType<boolean | import("../vc-table/interface").TableSticky>;
        default: any;
    };
    dropdownPrefixCls: StringConstructor;
    dataSource: {
        type: PropType<DefaultRecordType[]>;
        default: any;
    };
    pagination: {
        type: PropType<false | TablePaginationConfig>;
        default: any;
    };
    loading: {
        type: PropType<false | Partial<{
            prefixCls?: unknown;
            spinning?: unknown;
            size?: unknown;
            wrapperClassName?: unknown;
            tip?: unknown;
            delay?: unknown;
            indicator?: unknown;
        } & {
            prefixCls: string;
            delay: number;
            wrapperClassName: string;
            tip: string;
        } & {
            size?: "default" | "small" | "large";
            spinning?: boolean;
            indicator?: any;
        }>>;
        default: any;
    };
    size: {
        type: PropType<SizeType>;
        default: any;
    };
    bordered: BooleanConstructor;
    locale: {
        type: PropType<TableLocale>;
        default: any;
    };
    onChange: {
        type: PropType<(pagination: TablePaginationConfig, filters: Record<string, FilterValue | null>, sorter: SorterResult | SorterResult[], extra: TableCurrentDataSource) => void>;
        default: any;
    };
    onResizeColumn: {
        type: PropType<(w: number, col: ColumnType) => void>;
        default: any;
    };
    rowSelection: {
        type: PropType<TableRowSelection<DefaultRecordType>>;
        default: any;
    };
    getPopupContainer: {
        type: PropType<GetPopupContainer>;
        default: any;
    };
    scroll: {
        type: PropType<{
            x?: string | number | true;
            y?: string | number;
        } & {
            scrollToFirstRowOnChange?: boolean;
        }>;
        default: any;
    };
    sortDirections: {
        type: PropType<SortOrder[]>;
        default: any;
    };
    showSorterTooltip: {
        type: PropType<boolean | Partial<{
            title?: unknown;
            trigger?: unknown;
            visible?: unknown;
            defaultVisible?: unknown;
            placement?: unknown;
            color?: unknown;
            transitionName?: unknown;
            overlayStyle?: unknown;
            overlayClassName?: unknown;
            openClassName?: unknown;
            prefixCls?: unknown;
            mouseEnterDelay?: unknown;
            mouseLeaveDelay?: unknown;
            getPopupContainer?: unknown;
            arrowPointAtCenter?: unknown;
            autoAdjustOverflow?: unknown;
            destroyTooltipOnHide?: unknown;
            align?: unknown;
            builtinPlacements?: unknown;
            children?: unknown;
            onVisibleChange?: unknown;
            'onUpdate:visible'?: unknown;
        } & {
            prefixCls: string;
            color: string;
            align: {
                [key: string]: any;
            };
            transitionName: string;
            builtinPlacements: {
                [key: string]: any;
            };
            mouseEnterDelay: number;
            mouseLeaveDelay: number;
            overlayClassName: string;
            openClassName: string;
            children: unknown[];
        } & {
            title?: any;
            visible?: boolean;
            getPopupContainer?: (...args: any[]) => any;
            trigger?: "click" | "contextmenu" | "focus" | "hover" | ("click" | "contextmenu" | "focus" | "hover")[];
            "onUpdate:visible"?: (...args: any[]) => any;
            defaultVisible?: boolean;
            placement?: "left" | "right" | "bottom" | "top" | "bottomLeft" | "bottomRight" | "topLeft" | "topRight" | "leftTop" | "leftBottom" | "rightTop" | "rightBottom";
            overlayStyle?: import("vue").CSSProperties;
            destroyTooltipOnHide?: boolean;
            onVisibleChange?: (...args: any[]) => any;
            autoAdjustOverflow?: boolean | {
                [key: string]: any;
            };
            arrowPointAtCenter?: boolean;
        }>>;
        default: boolean;
    };
    contextSlots: {
        type: PropType<ContextSlots>;
    };
    transformCellText: {
        type: PropType<import("../vc-table/interface").TransformCellText<DefaultRecordType>>;
    };
};
declare const Table: import("vue").DefineComponent<TableProps<DefaultRecordType>, {}, {}, {}, {}, import("vue").ComponentOptionsMixin, import("vue").ComponentOptionsMixin, import("vue").EmitsOptions, string, import("vue").VNodeProps & import("vue").AllowedComponentProps & import("vue").ComponentCustomProps, Readonly<TableProps<DefaultRecordType>>, {}>;
export default Table;
